package symp.tradeworkflow;

import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.logging.Logger;

import symp.stardust.StardustUtil;

public class ApproverApp {
	static final Logger log = Logger.getLogger(ApproverApp.class.getName());
	static final String APPROVAL_KEY = "approvals";
	static final int APPROVAL_MAX = 2;
	Map<String, Object> approvals;

	public void nop() {
		log.info("nop()");
	}


	/**
	 * Make an Approval object with a Trade
	 * 
	 * @return
	 */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public void makeApprovals() {
		log.info("makeApprovals() begins");
		approvals = new LinkedHashMap<>();

		List approvalList = new ArrayList();
		for (int i = 1; i <= APPROVAL_MAX; ++i) {
			Map<String, Object> ai = new LinkedHashMap<>();
			ai.put("Approver", "APPROVER");
			ai.put("Approved", false);
			ai.put("Comment", "");
			ai.put("Level", i);
			approvalList.add(ai);
		}
		approvals.put(APPROVAL_KEY, approvalList);	
		log.info("makeApprovals() returns:\n" + StardustUtil.toString(approvals));
	}

	public Map<String, Object> getApprovals() {
		log.info("getApprovals() returns:\n" + StardustUtil.toString(approvals));
		return approvals;
	}

	public void setApprovals(Map<String, Object> approvals) {
		log.info("setApprovals( \n" + StardustUtil.toString(approvals) + "\n)");
		this.approvals = approvals;
	}

	@SuppressWarnings({ "rawtypes" })
	@Override
	public String toString() {

		StringBuilder sb = new StringBuilder();
		sb.append("ApprovalApp: ");
		boolean approved = false;

		if (approvals != null) {
			List approvalList = (List) approvals.get(APPROVAL_KEY);
			approved = true;
			for (int i = 0; i < approvalList.size(); ++i) {

				Map approval = (Map) approvalList.get(i);
				boolean lvlApproved = toBool(approval.get("Approved"), true);
				String approvedLabel = lvlApproved ? "Approved" : "Rejected";
				String approver = (String) approval.get("Approver");
				int level = toInteger(approval.get("Level"), 0);
				String comment = (String) approval.get("Comment");
				if (approver != null) {
					approved = approved && lvlApproved;
					sb.append(String.format("\tLevel %d %s by %s, comment: %s\n", level, approvedLabel, approver,
							comment));
				}
			}
		} else {
			sb.append("approvals is null");
		}
	
		return sb.toString();
	}
	
	public void log() {
		log.info( "log(): " + toString() );
	}

	private int toInteger(Object object, int i) {
		if (object instanceof Integer) {
			i = (Integer) object;
		} else if (object instanceof Long) {
			i = ((Long) object).intValue();
		}
		return i;
	}

	boolean toBool(Object o, boolean defaultValue) {
		if (o instanceof Boolean) {
			Boolean b = (Boolean) o;
			return b.booleanValue();
		}
		return defaultValue;
	}

}
